/*
 * RFBCopyRectangle.m
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

#import "RFB.h"

@implementation RFBCopyRectangle

- (id)initWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super initWithLocation:location encoding:encoding adaptor:adaptor]) {
        _source = NSMakePoint([[adaptor socket] readUnsignedShort],
                              [[adaptor socket] readUnsignedShort]);
    }

    return self;
}

- (NSPoint)source
{
    return _source;
}

- (void)drawWithFramebuffer:(RFBFramebuffer *)framebuffer
{
    [framebuffer copyPixelsFromRect:NSMakeRect(_source.x, _source.y,
                                               _location.size.width, _location.size.height)
                             toRect:_location];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@[0x%lx] location=%@ encoding=%d source=%@>",
        [[self class] description], self,
        NSStringFromRect(_location), _encoding, NSStringFromPoint(_source)];
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super initWithCoder:coder]) {
        _source = [coder decodePoint];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];
    [coder encodePoint:_source];
}

@end

