/*
 * RFBFramebuffer.h
 * A simple off-screen framebuffer optimized for drawing rectangles and raw images.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

@class NSImage;

extern unsigned RFBConvertPixel8ToPixel32(unsigned char pixel8);
extern unsigned RFBConvertPixel16ToPixel32(unsigned short pixel16);

@interface RFBFramebuffer : NSObject
{
    unsigned _width, _height;
    unsigned _bytesPerRow;
    NSRect _minimumUpdateRect;
    NSMutableData *_framebufferCover;
    unsigned *_framebufferPtr;
}

- (id)initWithSize:(NSSize)size;

- (NSImage *)framebufferImage;

- (void)compositePixelData:(NSData *)pixelData targetRect:(NSRect)target bitsPerPixel:(unsigned)bitsPerPixel;

- (NSData *)subPixelsInRect:(NSRect)rect;

- (void)copyPixelsFromRect:(NSRect)sourceRect toRect:(NSRect)destinationRect;

- (void)fillRect:(NSRect)rect color:(unsigned)color;
- (void)fillRectList:(const NSRect *)rectList colorList:(const unsigned *)colorList count:(unsigned)count;

- (void)update;
- (void)updateMinimum;

@end

