/*
 * RFBHextileRectangle.m
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

#import "RFB.h"

@implementation RFBHextileRectangle

- (id)initWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super initWithLocation:location encoding:encoding adaptor:adaptor]) {
        NSRect subRect;
        NSPoint maxPoint = NSMakePoint(_location.origin.x+_location.size.width,
                                       _location.origin.y+_location.size.height);

        _subrectangles = [[NSMutableArray alloc] init];

        for (subRect.origin.y = _location.origin.y; subRect.origin.y < maxPoint.y;
             subRect.origin.y += 16) {
            for (subRect.origin.x = _location.origin.x; subRect.origin.x < maxPoint.x;
                 subRect.origin.x += 16) {
                subRect.size.width = 16;
                subRect.size.height = 16;

                if (maxPoint.y - subRect.origin.y < 16)
                    subRect.size.height = maxPoint.y - subRect.origin.y;
                if (maxPoint.x - subRect.origin.x < 16)
                    subRect.size.width = maxPoint.x - subRect.origin.x;

                [_subrectangles addObject:[RFBHextileSubrectangle subrectangleWithLocation:subRect superrectangle:self adaptor:adaptor]];
            }
        }
    }

    return self;
}

- (void)dealloc
{
    [_subrectangles release];

    [super dealloc];
}

- (void)setLastBackgroundColor:(unsigned)color
{
    _lastBackgroundColor = color;
}

- (unsigned)lastBackgroundColor
{
    return _lastBackgroundColor;
}

- (void)drawWithFramebuffer:(RFBFramebuffer *)framebuffer
{
    NSEnumerator *rectEnumerator = [_subrectangles objectEnumerator];
    RFBHextileSubrectangle *subrect;

    while (subrect = [rectEnumerator nextObject])
        [subrect drawWithFramebuffer:framebuffer];
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super initWithCoder:coder]) {
        [coder decodeValueOfObjCType:@encode(unsigned) at:&_lastBackgroundColor];
        _subrectangles = [[coder decodeObject] retain];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    [coder encodeValueOfObjCType:@encode(unsigned) at:&_lastBackgroundColor];
    [coder encodeObject:_subrectangles];
}

@end


