/*
 * RFBMessage.h
 * The RFB message class cluster.
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

@class NSData, NSString;
@class TCStream;
@class RFBProtocolAdaptor;

#define RFBFramebufferUpdate		0
#define RFBSetColourMapEntries		1
#define RFBBell				2
#define RFBServerCutText		3

@interface RFBMessage : NSObject <NSCopying, NSCoding>
{
    octet _type;
}

+ (RFBMessage *)messageWithAdaptor:(RFBProtocolAdaptor *)adaptor;

+ (RFBMessage *)framebufferUpdateMessageWithRectangles:(NSArray *)rectangles;
+ (RFBMessage *)bellMessage;
+ (RFBMessage *)cutTextMessageWithText:(NSString *)text;

- (id)initWithType:(octet)type;

- (octet)type;

@end

@interface RFBFramebufferUpdateMessage : RFBMessage <NSCoding>
{
    NSArray *_rectangles;
}

- (id)initWithRectangles:(NSArray *)rectangles;

- (NSArray *)rectangles;

@end

@interface RFBCutTextMessage : RFBMessage <NSCoding>
{
    NSString *_text;
}

- (id)initWithText:(NSString *)text;

- (NSString *)text;

@end
