/*
 * RFBProtocolAdaptor.h
 *
 *
 * Copyright (c) 1997 21st Century Software#defineNew York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file#defineand for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

@class NSString, NSData;

extern NSString *RFBProtocolVersion;
extern NSString *RFBProtocolException;

#define RFBConnectionFailed		0
#define RFBNoAuthorization		1
#define RFBVNCAuthorization		2

#define RFBAuthorizationOK		0
#define RFBAuthorizationFailed		1
#define RFBAuthorizationTooMany 	2

#define RFBSetPixelFormat		0
#define RFBFixColourMapEntries		1
#define RFBSetEncodings			2
#define RFBFramebufferUpdateRequest	3
#define RFBKeyEvent			4
#define RFBPointerEvent			5
#define RFBClientCutText		6

#define RFBFlashPort  (5400)    /* Offset to listen for `flash' commands */
#define RFBClientPort (5500)    /* Offset to listen for reverse connections */
#define RFBServerPort (5900)    /* Offset to server for regular connections */

@class NSString;
@class RFBMessage;

@interface RFBProtocolAdaptor : NSObject
{
    @private
    NSString *_host;
    int _port;
    TCSocket *_socket;
    
    unsigned short _width;
    unsigned short _height;
    octet _bitsPerPixel;
    octet _depth;
    BOOL _bigEndian;
    BOOL _trueColour;
    unsigned short _redMax;
    unsigned short _greenMax;
    unsigned short _blueMax;
    octet _redShift;
    octet _greenShift;
    octet _blueShift;
    NSString *_desktopName;

    @public
        int _versionMajor, _versionMinor;
}

+ (RFBProtocolAdaptor *)adaptorWithHost:(NSString *)host port:(int)port;

- (id)initWithHost:(NSString *)host port:(int)port;

- (TCSocket *)socket;
- (void)abort;

- (unsigned short)width;
- (unsigned short)height;
- (octet)bitsPerPixel;
- (octet)depth;
- (BOOL)bigEndian;
- (BOOL)trueColour;
- (unsigned short)redMax;
- (unsigned short)greenMax;
- (unsigned short)blueMax;
- (octet)redShift;
- (octet)greenShift;
- (octet)blueShift;

- (NSString *)desktopName;

- (int)versionMajor;
- (int)versionMinor;

// Protocol methods
- (NSString *)readServerVersionMessage;
- (void)sendClientVersionMessage;

- (int)readServerAuthScheme;
- (NSData *)readServerAuthChallenge;
- (void)readServerInitializationMessage;

- (BOOL)authorizeWithPassword:(NSString *)passString challenge:(NSData *)challenge;

- (void)sendClientInitializationWithSharedDesktop:(BOOL)sharedFlag;
- (void)sendEncodings:(NSArray *)encodings;
- (void)sendBitsPerPixel:(octet)bitsPerPixel depth:(octet)depth bigEndian:(octet)bigEndian trueColour:(octet)trueColour redMax:(unsigned short)redMax greenMax:(unsigned short)greenMax blueMax:(unsigned short)blueMax redShift:(octet)redShift greenShift:(octet)greenShift blueShift:(octet)blueShift;
- (void)sendFramebufferUpdateRequestForRect:(NSRect)r incremental:(BOOL)incremental;

- (void)sendPointerEventWithButtonMask:(octet)buttonMask location:(NSPoint)location;
- (void)sendKeyEventWithPressedFlag:(BOOL)isPressed key:(unsigned)key;

- (unsigned)read24BitColor;

@end

