/*
 * RFBRRERectangle.m
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

#import "RFB.h"

@interface RFBRRERectangle(Private)
- (void)_readRectanglesWithAdaptor:(RFBProtocolAdaptor *)adaptor;
@end

@implementation RFBRRERectangle

- (id)initWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super initWithLocation:location encoding:encoding adaptor:adaptor]) {
        _numberOfRects = [[adaptor socket] readUnsignedInt];
        _backgroundColor = [adaptor read24BitColor];

        _rectListCover = [[NSMutableData alloc] initWithLength:(_numberOfRects*sizeof(NSRect))];
        _colorListCover = [[NSMutableData alloc] initWithLength:(_numberOfRects*sizeof(unsigned))];

        [self _readRectanglesWithAdaptor:adaptor];
    }

    return self;
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@[0x%lx] location=%@ encoding=%d numberOfRects=%d backgroundColor=0x%08lx>",
        [[self class] description], self,
        NSStringFromRect(_location), _encoding, _numberOfRects, _backgroundColor];
}

- (void)_readRectanglesWithAdaptor:(RFBProtocolAdaptor *)adaptor
{
    NSRect *rectList = [_rectListCover mutableBytes];
    unsigned *colorList = [_colorListCover mutableBytes];
    unsigned i;

    for (i = 0; i < _numberOfRects; ++i) {
        colorList[i] = [adaptor read24BitColor];
        rectList[i].origin.x = _location.origin.x + [[adaptor socket] readUnsignedShort];
        rectList[i].origin.y = _location.origin.y + [[adaptor socket] readUnsignedShort];
        rectList[i].size.width = [[adaptor socket] readUnsignedShort];
        rectList[i].size.height = [[adaptor socket] readUnsignedShort];
    }
}

- (void)dealloc
{
    [_rectListCover release];
    [_colorListCover release];

    [super dealloc];
}

- (void)drawWithFramebuffer:(RFBFramebuffer *)framebuffer
{
    [framebuffer fillRect:_location color:_backgroundColor];
    [framebuffer fillRectList:[_rectListCover bytes]
                    colorList:[_colorListCover bytes]
                        count:_numberOfRects];
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super initWithCoder:coder]) {
        [coder decodeValueOfObjCType:@encode(unsigned) at:&_backgroundColor];
        [coder decodeValueOfObjCType:@encode(unsigned) at:&_numberOfRects];

        _rectListCover = [[coder decodeDataObject] retain];
        _colorListCover = [[coder decodeDataObject] retain];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    [coder encodeValueOfObjCType:@encode(unsigned) at:&_backgroundColor];
    [coder encodeValueOfObjCType:@encode(unsigned) at:&_numberOfRects];
    [coder encodeDataObject:_rectListCover];
    [coder encodeDataObject:_colorListCover];
}

@end

