/*
 * RFBRawRectangle.m
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

#import "RFB.h"

@implementation RFBRawRectangle

- (id)initWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super initWithLocation:location encoding:encoding adaptor:adaptor]) {
        unsigned imageLength = location.size.width * location.size.height * ([adaptor bitsPerPixel]/8);
        _bitsPerPixel = [adaptor bitsPerPixel];
        _pixelData = [[[adaptor socket] readDataOfLength:imageLength] retain];
    }

    return self;
}

- (void)drawWithFramebuffer:(RFBFramebuffer *)framebuffer
{
    [framebuffer compositePixelData:_pixelData
                         targetRect:_location
                       bitsPerPixel:_bitsPerPixel];
}

- (void)dealloc
{
    [_pixelData release];

    [super dealloc];
}

- (NSData *)pixelData
{
    return _pixelData;
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@[0x%lx] location=%@ encoding=%d pixelData length=%u>",
        [[self class] description], self,
        NSStringFromRect(_location), _encoding, [_pixelData length]];
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super initWithCoder:coder]) {
        [coder decodeValueOfObjCType:@encode(unsigned) at:&_bitsPerPixel];
        _pixelData = [[coder decodeDataObject] retain];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    [coder encodeValueOfObjCType:@encode(unsigned) at:&_bitsPerPixel];
    [coder encodeDataObject:_pixelData];
}

@end
