/*
 * RFBRectangle.m
 *
 *
 * Copyright (c) 1997 21st Century Software, New York City.
 *
 * See the file Reference/License.rtf for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * Written by David Young <dwy@ace.net>, 1997.
 */

#import "RFB.h"

@implementation RFBRectangle

+ (RFBRectangle *)rectangleWithAdaptor:(RFBProtocolAdaptor *)adaptor
{
    TCStream *stream = [adaptor socket];
    NSRect r;
    unsigned encoding = 0;

    r.origin.x = [stream readUnsignedShort];
    r.origin.y = [stream readUnsignedShort];
    r.size.width = [stream readUnsignedShort];
    r.size.height = [stream readUnsignedShort];

    encoding = [stream readUnsignedInt];
    
    switch(encoding) {
        case RFBEncodingRaw:
            return [RFBRawRectangle rectangleWithLocation:r encoding:encoding adaptor:adaptor];

        case RFBEncodingCopyRect:
            return [RFBCopyRectangle rectangleWithLocation:r encoding:encoding adaptor:adaptor];

        case RFBEncodingRRE:
            return [RFBRRERectangle rectangleWithLocation:r encoding:encoding adaptor:adaptor];

        case RFBEncodingCoRRE:
            return [RFBCoRRERectangle rectangleWithLocation:r encoding:encoding adaptor:adaptor];

        case RFBEncodingHextile:
            return [RFBHextileRectangle rectangleWithLocation:r encoding:encoding adaptor:adaptor];

        default:
            [NSException raise:NSInvalidArgumentException
                        format:@"%@: unknown encoding method %d",
                NSStringFromSelector(_cmd), encoding];
    }

    return nil; // not reached
}

+ (RFBRectangle *)rectangleWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    return [[[self alloc] initWithLocation:location encoding:encoding adaptor:adaptor] autorelease];
}

- (id)initWithLocation:(NSRect)location encoding:(unsigned)encoding adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super init]) {
        _location.origin.x = location.origin.x;
        _location.origin.y = location.origin.y;
        _location.size.width = location.size.width;
        _location.size.height = location.size.height;

        _encoding = encoding;
    }

    return self;
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"<%@[0x%lx] location=%@ encoding=%d>",
        [[self class] description], self,
        NSStringFromRect(_location), _encoding];
}

- (void)drawWithFramebuffer:(RFBFramebuffer *)framebuffer
{
}

- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}

- (NSRect)location
{
    return _location;
}

- (unsigned)encoding
{
    return _encoding;
}

- (id)initWithCoder:(NSCoder *)coder
{
    if (self = [super init]) {
        _location = [coder decodeRect];
        [coder decodeValueOfObjCType:@encode(unsigned) at:&_encoding];
    }

    return self;
}

- (void)encodeWithCoder:(NSCoder *)coder
{
    [coder encodeRect:_location];
    [coder encodeValueOfObjCType:@encode(unsigned) at:&_encoding];
}

@end
