#import "VNC.h"

@implementation VNCInputView

- (id)initWithFrame:(NSRect)frame adaptor:(RFBProtocolAdaptor *)adaptor
{
    if (self = [super initWithFrame:frame]) {
        _mouseMask = 0;
        _adaptor = [adaptor retain];
    }

    return self;
}

- (void)dealloc
{
    [_adaptor release];

    [super dealloc];
}

- (BOOL)isFlipped
{
    return YES;
}

- (BOOL)isOpaque
{
    return NO;
}

- (BOOL)acceptsFirstResponder
{
    return YES;
}

- (void)mouseDown:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    _mouseMask |= 0x01;
    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)mouseUp:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    _mouseMask &= ~(0x01);
    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)rightMouseDown:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    _mouseMask |= 0x02;
    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)rightMouseUp:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    _mouseMask &= ~(0x02);
    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)mouseMoved:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    if ([self mouse:newLoc inRect:[self frame]])
        [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)mouseDragged:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)rightMouseDragged:(NSEvent *)theEvent
{
    NSPoint newLoc = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    [_adaptor sendPointerEventWithButtonMask:_mouseMask location:newLoc];
}

- (void)mouseEntered:(NSEvent *)theEvent
{
    [[self window] setAcceptsMouseMovedEvents:YES];
    [[VNCController hollowCursor] push];
}

- (void)mouseExited:(NSEvent *)theEvent
{
    [[self window] setAcceptsMouseMovedEvents:NO];
    [NSCursor pop];
}

- (void)keyDown:(NSEvent *)theEvent
{
    NSData *data = [[theEvent charactersIgnoringModifiers] dataUsingEncoding:NSASCIIStringEncoding];
    const octet *charPtr = [data bytes];
    unsigned i = 0;

    if ([theEvent modifierFlags] & NSShiftKeyMask)		// no left/right distinction
        [_adaptor sendKeyEventWithPressedFlag:YES key:0xffe1];
    if ([theEvent modifierFlags] & NSControlKeyMask)
        [_adaptor sendKeyEventWithPressedFlag:YES key:0xffe3];
    if ([theEvent modifierFlags] & NSCommandKeyMask)	// command is mapped to META
        [_adaptor sendKeyEventWithPressedFlag:YES key:0xffe7];
    if ([theEvent modifierFlags] & NSAlternateKeyMask)
        [_adaptor sendKeyEventWithPressedFlag:YES key:0xffe9];
       
    for (i = 0; i < [data length]; ++i)
        [_adaptor sendKeyEventWithPressedFlag:YES key:(unsigned)charPtr[i]];
}

- (void)keyUp:(NSEvent *)theEvent
{
    NSData *data = [[theEvent charactersIgnoringModifiers] dataUsingEncoding:NSASCIIStringEncoding];
    const octet *charPtr = [data bytes];
    unsigned i = 0;

    if ([theEvent modifierFlags] & NSShiftKeyMask)		// no left/right distinction
        [_adaptor sendKeyEventWithPressedFlag:NO key:0xffe1];
    if ([theEvent modifierFlags] & NSControlKeyMask)
        [_adaptor sendKeyEventWithPressedFlag:NO key:0xffe3];
    if ([theEvent modifierFlags] & NSCommandKeyMask)	// command is mapped to META
        [_adaptor sendKeyEventWithPressedFlag:NO key:0xffe7];
    if ([theEvent modifierFlags] & NSAlternateKeyMask)
        [_adaptor sendKeyEventWithPressedFlag:NO key:0xffe9];

    for (i = 0; i < [data length]; ++i)
        [_adaptor sendKeyEventWithPressedFlag:NO key:(unsigned)charPtr[i]];
}

@end
